"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const expression_1 = require("./expression");
const parameter_modifier_1 = require("./parameter-modifier");
const variable_1 = require("./variable");
const linq_1 = require("@azure-tools/linq");
/** represents a method parameter */
class Parameter extends variable_1.Variable {
    constructor(name, type, objectInitializer) {
        super();
        this.name = name;
        this.type = type;
        this.description = '';
        this.genericParameters = new Array();
        this.params = false;
        this.modifier = parameter_modifier_1.ParameterModifier.None;
        this.attributes = new Array();
        this.apply(objectInitializer);
        if (!this.description.trim()) {
            this.description = '';
        }
    }
    get attributeDeclaration() {
        return linq_1.length(this.attributes) > 0 ? `${this.attributes.joinWith(each => `${each.value}`, ' ')} ` : '';
    }
    get comment() {
        return `<param name="${this.name}">${this.description}</param>`;
    }
    get declaration() {
        if (this.defaultInitializer) {
            return `${this.params ? 'params ' : ''}${this.attributeDeclaration}${this.modifier} ${this.type.declaration} ${this.name} = ${this.defaultInitializer.value}`.trim();
        }
        return `${this.params ? 'params ' : ''}${this.attributeDeclaration}${this.modifier} ${this.type.declaration} ${this.name}`.trim();
    }
    get use() {
        return this.name;
    }
    get value() {
        return `${this.name}`;
    }
    assign(expression) {
        return `${this.name} = ${expression_1.valueOf(expression)};`;
    }
    assignPrivate(expression) {
        return this.assign(expression);
    }
    get declarationExpression() {
        return this;
    }
    get declarationStatement() {
        throw new Error('Property can not be a declaration statement');
    }
}
exports.Parameter = Parameter;
//# sourceMappingURL=parameter.js.map