import { DeepPartial } from '@azure-tools/codegen';
import { Dictionary } from '@azure-tools/linq';
import { Abstract, Access, Extern, New, Override, Sealed, Static, Virtual } from './access-modifier';
import { Attribute } from './attribute';
import { Expression, ExpressionOrLiteral } from './expression';
import { OneOrMoreStatements, Statement, StatementPossibilities } from './statements/statement';
import { TypeDeclaration } from './type-declaration';
import { ExpressionStatement, Instance, Variable } from './variable';
export declare class Property extends Variable implements Instance {
    name: string;
    type: TypeDeclaration;
    'new': New;
    getAccess: Access;
    setAccess: Access;
    'static': Static;
    virtual: Virtual;
    sealed: Sealed;
    override: Override;
    abstract: Abstract;
    extern: Extern;
    attributes: Attribute[];
    metadata: Dictionary<any>;
    description: string;
    get?: StatementPossibilities | Expression;
    set?: StatementPossibilities | Expression;
    protected get visibility(): Access;
    add<T extends object>(item: T & (Attribute)): T;
    protected get attributeDeclaration(): string;
    constructor(name: string, type: TypeDeclaration, objectInitializer?: DeepPartial<Property>);
    protected get getterDeclaration(): string;
    protected get setterDeclaration(): string;
    protected get getter(): string;
    protected get setter(): string;
    get declaration(): string;
    get value(): string;
    get valuePrivate(): string;
    assign(expression: ExpressionOrLiteral): OneOrMoreStatements;
    assignPrivate(expression: ExpressionOrLiteral): OneOrMoreStatements;
    get declarationExpression(): Expression;
    get declarationStatement(): Statement;
    invokeMethod(methodName: string, ...parameters: Array<Expression>): ExpressionStatement;
}
export declare class Indexer extends Property {
    keyType: TypeDeclaration;
    constructor(keyType: TypeDeclaration, valueType: TypeDeclaration, objectInitializer: Partial<Indexer>);
    get index(): string;
}
export declare class LambdaProperty extends Property {
    name: string;
    type: TypeDeclaration;
    expression: Expression;
    constructor(name: string, type: TypeDeclaration, expression: Expression, objectInitializer?: DeepPartial<LambdaProperty>);
    get declaration(): string;
}
export declare class LazyProperty extends Property {
    name: string;
    type: TypeDeclaration;
    expression: Expression;
    private backingName;
    instanceAccess: string;
    constructor(name: string, type: TypeDeclaration, expression: Expression, objectInitializer?: DeepPartial<LazyProperty>);
    get declaration(): string;
}
export declare class BackedProperty extends Property {
    backingName: string;
    initializer?: ExpressionOrLiteral;
    constructor(name: string, type: TypeDeclaration, objectInitializer?: DeepPartial<BackedProperty>);
    get declaration(): string;
    get value(): string;
    get valuePrivate(): string;
    assignPrivate(expression: ExpressionOrLiteral): OneOrMoreStatements;
}
//# sourceMappingURL=property.d.ts.map